
/* JavaScript content from js/ui/ui-base.js in folder common */
var uiBase = {
	uiClass: 'ui-base',					// script default class
	uiWrappingId : 'ui-base-wrap',		// all wrapping element id
	uiActive: 'active',					// script active class
	uiTarget: 'ui-base-target',
	uiSplit: "-",						// script class split sign 
	uiControlArray: [],
	uiContent: 'content',				// script content class
	uiActionBase: ['toggle', 'tabs', 'layer'],
	
	/**
	 * UI 객체생성
	 * 생성되는 객체들에만 공통 UI 관련 함수가 적용된다.
	 * @elemObj		: 실제로 바인딩이 될 엘리먼트 객체
	 * @elemClass	: 생성된 객체의 class 배열
	 * @targetElem	: 객체의 타켓 객체
	 */
	uiControlObj: function(elemObj, elemClass, targetElem) {
		this.elemObj = elemObj;
		this.elemClass = elemClass;
		this.targetElem = targetElem;
	},
	
	/**
	 * uiClass 스크립트 적용 항목을 찾아 uiControlObj 객체를 각각 생성하여 uiControlArray에 배열에 추가
	 */
	_find: function() {
		$(uiBase.uiActionBase).each(function(e) {
			$('.' + uiBase.uiClass + uiBase.uiSplit + uiBase.uiActionBase[e]).each(function() {
				var elemObj = this;
				var classList = $(this).attr('class').split(/\s+/);
				var elemClass = [];
				$(classList).each(function(e) {
					elemClass.push(this);
				});
				uiBase.uiControlArray.push(new uiBase.uiControlObj(elemObj, elemClass));
			});
		});
	},
	
	
	/**
	 * uiControlArray 배열을 각각 순회하며 action 함수 호출
	 */
	_binding: function() {
		
		$(uiBase.uiControlArray).each(function(e) {
			
			
			var elemObj = $(this.elemObj);
			var target = $(this.elemObj).attr('href');
			
			uiBase.uiControlArray[e].targetElem = target;
			
			$(this.elemClass).each(function(e) {
				//toggle action
				if(this == uiBase.uiClass + uiBase.uiSplit + uiBase.uiActionBase[0]) {
					if (target && target.length > 0 && target.charAt(0) == '#') {
						$(target).addClass(uiBase.uiTarget);
					}
					
					if($(elemObj).hasClass(uiBase.uiClass + uiBase.uiSplit + "hover")) {
						uiBase._action(elemObj, target, "toggle", "hover");
					} else if($(elemObj).hasClass("ui-dimmed")) {
						uiBase._action(elemObj, target, "toggle", "dimmed");
					} else {
						uiBase._action(elemObj, target, "toggle", "click");
					}
				}
				//tab action
				if(this == uiBase.uiClass + uiBase.uiSplit + uiBase.uiActionBase[1]) {
					if($(elemObj).hasClass(uiBase.uiClass + uiBase.uiSplit + "hover")) {
						uiBase._action(elemObj, target, "tabs", "hover");
					} else {
						uiBase._action(elemObj, target, "tabs", "click");
					}
				}
			});
		});
	},
	
	/**
	 * 호출된 파라메타 정보로, 각 객체에 해당하는 이벤트를 실제 바인딩
	 * @obj		: 실제로 바인딩이 될 엘리먼트 객체
	 * @target	: 이벤트의 영향을 받을 타켓 엘리먼트 객체
	 * @act		: 수행될 기능
	 * @dimmed	: 딤드여부 확인
	 */
	_action: function(obj, target, act, evt) {
		
		/**
		 * 토글 기능
		 * hover 또는 click 이벤트 발생시 해당 엘리먼트와 타겟이 되는 엘리먼트에 uiBase.uiActive 를 토글해 준다.
		 */
		if(act == "toggle" && evt == "hover") {
			$(obj).hover(function() {
				$(this).addClass(uiBase.uiActive);
				$(target).addClass(uiBase.uiActive);
				}, function() {
				$(this).removeClass(uiBase.uiActive);
				$(target).removeClass(uiBase.uiActive);
			});
			$(obj).click(function(e) {
				e.preventDefault();
			});
		}
		if(act == "toggle" && evt == "click") {
			$(obj).click(function(e) {
				e.preventDefault();
				$(this).toggleClass(uiBase.uiActive);
				$(target).toggleClass(uiBase.uiActive);
			});
		}
		
		if(act == "toggle" && evt == "dimmed") {
			uiBase._dimmed(obj);
		}
		
		/**
		 * 탭 기능
		 * hover 또는 click 이벤트 발생시 해당 엘리먼트와 타겟이 되는 엘리먼트에 uiBase.uiActive 를 토글해 주며 해당하는 컨텐츠 영역을 보여주고, 해당되지 않는 영역은 숨긴다.
		 */
		if(act == "tabs" && evt == "hover") {
			$(obj).find('li').mouseover(function(e) {
				e.preventDefault();
				var target = $(this).find('a').attr('href');
				$(this).addClass(uiBase.uiActive).siblings('li').removeClass(uiBase.uiActive);
				$(target).addClass(uiBase.uiActive).siblings('.'+uiBase.uiContent).removeClass(uiBase.uiActive);
			});
			$(obj).click(function(e) {
				e.preventDefault();
			});
		}
		if(act == "tabs" && evt == "click") {
			$(obj).find('li').click(function(e) {
				
				e.preventDefault();
				
				var target = $(this).find('a').attr('href');
				$(this).addClass(uiBase.uiActive).siblings('li').removeClass(uiBase.uiActive);
			});
		}
	},
	
	_layerPopup: function (){
		$('.ui-base-layer').each(function(){
			var target = $(this).attr('href');
			
			if (target.indexOf("javascript") > -1){
				return ;
			}
			
			var closebtn = $(target).find('.close-layer');
			var dim = $('#dimmedAll');
			
			$(this).click(function(e){
				e.preventDefault();
				$(target).addClass(uiBase.uiActive);
				dim.addClass(uiBase.uiActive);
				$(document).unbind("touchmove");
				// 하단 Util 영역 숨김
				//mainCtrl.hideTabBasicUtil();
				
				var con_height = $(window).height();
				var obj_height = $(target).height();
				var center_position = parseInt((con_height - obj_height)/4);
				
				if(obj_height > con_height){
					$(target).css({
						'top': $(window).scrollTop() + 'px'
					});
				} else {
					$(target).css({
						'top': ($(window).scrollTop() + center_position) + 'px'
					});
				}
			});
			
			dim.click(function(){
				$(target).removeClass(uiBase.uiActive);
				$(this).removeClass(uiBase.uiActive);
			});
			
			closebtn.click(function(e){
				e.preventDefault();
				$(target).removeClass(uiBase.uiActive);
				dim.removeClass(uiBase.uiActive);
				// 하단 Util 영역 표시
				mainCtrl.showTabBasicUtil();
			});
			
			/*$(window).resize(function(){
				var con_height = parseInt($(window).height()/2);
				var obj_height = parseInt(($(target).height() + 35 + 20) / 2);
				var center_position = con_height - obj_height;
				
				if(obj_height > con_height){
					$(target).css({
						'top': $(window).scrollTop() + 'px'
					});
				} else {
					$(target).css({
						'top': ($(window).scrollTop() + center_position) + 'px'
					});
				}
			});*/
			
		});
		
	},
	
	/*
	 */
	_CreateDimmed: function() {
		if ($("#dimmedAll").length > 0) {
			$(document).bind('touchmove', function(e) {
				if ($('#dimmedAll').hasClass('active') && !$("section.layer-popup").hasClass('active')) {
					e.preventDefault();
				}
			});
		} else {
			var dimmedAll = $('<div class="dimmed-all" id="dimmedAll"></div>');
			$('#container').append(dimmedAll);
		}
	},
	
	/*
	 */
	_dimmed: function(elem) {
		var dim = $("#dimmedAll");
		var obj = elem;
		var target = obj.attr("href");
		
		dim.click(function() {
			$(obj).removeClass(uiBase.uiActive);
			if (target && target.length > 0 && target.charAt(0) == '#') {
				$(target).removeClass(uiBase.uiActive);
			}
			if( $("#aside").hasClass("active") )
				$(dim).removeClass(uiBase.uiActive);
		});
		
		obj.click(function(e) {
			e.preventDefault();
			$(obj).toggleClass(uiBase.uiActive);
			if (target && target.length > 0 && target.charAt(0) == '#') {
				$(target).toggleClass(uiBase.uiActive);
			}
			$(dim).toggleClass(uiBase.uiActive);
		});
	},
	
	/*
	 */
	_init: function() {
		uiBase._CreateDimmed();
		uiBase._find();
		uiBase._binding();
		//uiBase._layerPopup();
	}
};
//uiBase._init();



